//#include <stm32f1/stm32f10x.h>     // definition uint32_t
#include <libopencm3/cm3/common.h> // BEGIN_DECL
#include <errno.h>
#include <stdio.h>
#include <unistd.h>

#include <libopencm3/cm3/nvic.h>
#include <libopencm3/stm32/gpio.h>
#include <libopencm3/stm32/usart.h>
#include <libopencm3/stm32/f1/rcc.h>
#include <libopencm3/stm32/f1/adc.h>

int _write(int file, char *ptr, int len);

#define nbr 256

static void clock_setup(void)
{ rcc_clock_setup_in_hse_8mhz_out_24mhz();

  rcc_periph_clock_enable(RCC_GPIOC);   // LED
  rcc_periph_clock_enable(RCC_GPIOA);   // RS32 sur port A
  rcc_periph_clock_enable(RCC_USART1);  // USART
  rcc_periph_clock_enable(RCC_ADC1);    // ADC

}

static void usart_setup(void)
{ gpio_set_mode(GPIOA, GPIO_MODE_OUTPUT_50_MHZ,
    GPIO_CNF_OUTPUT_ALTFN_PUSHPULL, GPIO_USART1_TX);

  usart_set_baudrate(USART1, 115200);
  usart_set_databits(USART1, 8);
  usart_set_stopbits(USART1, USART_STOPBITS_1);
  usart_set_mode(USART1, USART_MODE_TX);
  usart_set_parity(USART1, USART_PARITY_NONE);
  usart_set_flow_control(USART1, USART_FLOWCONTROL_NONE);
  usart_enable(USART1);
}

static void adc_setup(void)
{ int i;
  gpio_set_mode(GPIOA, GPIO_MODE_INPUT, GPIO_CNF_INPUT_ANALOG, GPIO0|GPIO1|GPIO2);
  adc_power_off(ADC1);
  adc_disable_scan_mode(ADC1);           // single conversion
  adc_set_single_conversion_mode(ADC1);
  adc_disable_external_trigger_regular(ADC1);
  adc_set_right_aligned(ADC1);
  adc_set_sample_time_on_all_channels(ADC1, ADC_SMPR_SMP_28DOT5CYC);
  adc_power_on(ADC1);
  for (i = 0; i < 800000; i++) __asm__("nop"); // demarrage ADC ...
  adc_reset_calibration(ADC1);
  adc_calibrate(ADC1);
}

static uint16_t read_adc_naiive(uint8_t channel)
{ uint8_t channel_array[16];
  channel_array[0] = channel;                 // choix du canal de conversion
  adc_set_regular_sequence(ADC1, 1, channel_array);
  adc_start_conversion_direct(ADC1);
  while (!adc_eoc(ADC1));                     // attend la fin de conversion
  return(adc_read_regular(ADC1));
}

int main(void)
{int i,tab0[nbr],tab1[nbr],tab2[nbr];
 clock_setup();
 usart_setup();
 gpio_set_mode(GPIOC, GPIO_MODE_OUTPUT_2_MHZ, GPIO_CNF_OUTPUT_PUSHPULL, GPIO8);
 gpio_set(GPIOC, GPIO8); 
 adc_setup();
 
 while (1) {
      for (i=0;i<nbr;i++) tab0[i]= read_adc_naiive(0);
      for (i=0;i<nbr;i++) tab1[i]= read_adc_naiive(1);
      for (i=0;i<nbr;i++) tab2[i]= read_adc_naiive(2);
      for (i=0;i<nbr;i++) printf("%d %d %d\n", tab0[i],tab1[i],tab2[i]);
      gpio_toggle(GPIOC, GPIO8); /* LED on/off */
      for (i = 0; i < 100000; i++) __asm__("NOP");
 }
 return 0;
}

int _write(int file, char *ptr, int len)
{ int i;
  if (file == STDOUT_FILENO || file == STDERR_FILENO) {
     for (i = 0; i < len; i++) {
         if (ptr[i] == '\n') {usart_send_blocking(USART1, '\r');}
         usart_send_blocking(USART1, ptr[i]);
       }
      return i;
  }
  errno = EIO;
  return -1;
}
