#include <libopencm3/cm3/common.h> // BEGIN_DECL
#include <libopencm3/stm32/f1/memorymap.h>
#include <libopencm3/stm32/f1/rcc.h>
#include <libopencm3/stm32/f1/gpio.h>
#include <libopencm3/stm32/timer.h>
#include <libopencm3/cm3/nvic.h>
#include <libopencm3/stm32/exti.h>

static void clock_setup(void) {rcc_clock_setup_in_hse_8mhz_out_24mhz();} // max 24 MHz

static void gpio_setup(void)
{ rcc_periph_clock_enable(RCC_GPIOC);   // LED
  gpio_set_mode(GPIOC, GPIO_MODE_OUTPUT_50_MHZ,
                GPIO_CNF_OUTPUT_PUSHPULL, GPIO8|GPIO9);          // LED output
  gpio_set(GPIOC, GPIO9);                                        // initial state
  gpio_clear(GPIOC, GPIO8);
}

static void tim_setup(void)
{ rcc_periph_clock_enable(RCC_TIM2);                            // TIM2 clock
  nvic_enable_irq(NVIC_TIM2_IRQ);                                // TIM2 interrupt
  timer_reset(TIM2);                                             // Reset TIM2 

  timer_set_mode(TIM2, TIM_CR1_CKD_CK_INT, TIM_CR1_CMS_EDGE, TIM_CR1_DIR_UP);
  timer_set_prescaler(TIM2, 90);          // prescaler value. 
  timer_disable_preload(TIM2);            // Enable preload
  timer_continuous_mode(TIM2);            // Continous mode
  timer_set_period(TIM2, 65535);          // Period 
  timer_disable_preload(TIM2);            // ARR reload enable
  timer_enable_counter(TIM2);             // Counter enable. 
  timer_enable_irq(TIM2, TIM_DIER_CC1IE); // Enable commutation interrupt. 
}

void tim2_isr(void)   // isr = Interrupt Service Routine, fonction de gestion de l'interruption
{ if (timer_get_flag(TIM2, TIM_SR_CC1IF)) {
     timer_clear_flag(TIM2, TIM_SR_CC1IF);
     gpio_toggle(GPIOC, GPIO8 | GPIO9);
    }
}

int main(void)
{ clock_setup();
  gpio_setup();
  tim_setup();
  while (1) __asm("nop"); // ce programme ne fait ... rien !
  return 0;
}
